# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2008 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import fields, osv
from tools import config
import time
from tools.translate import _

class account_payment_term(osv.osv):
    _name = "account.payment.term"
    _inherit = "account.payment.term"
    _columns = {
        'cash_discount_ids': fields.one2many('account.cash.discount', 'payment_id', 'Cash Discounts'),
    }
account_payment_term()

class account_cash_discount(osv.osv):
    _name = "account.cash.discount"
    _description = "Cash Discount" #A reduction in the price  if payment is made within a stipulated period.
    _order ="delay"
    _columns = {
        'name': fields.char('Name', size=32),
        'delay': fields.integer('Number of Days', required=True),
        'discount': fields.float('Discount (%)', digits=(16,6),required=True),
        'discount_account_id': fields.many2one('account.account', 'Customer Discount Account',required=True),
        'payment_id': fields.many2one('account.payment.term','Associated Payment Term'),
        'credit_account_id': fields.many2one('account.account', 'Credit Account'),
        'debit_account_id': fields.many2one('account.account', 'Debit Account'),
        'sup_discount_account_id': fields.many2one('account.account', 'Supplier Discount Account',required=True),
    }
account_cash_discount()

class account_payment_term_line(osv.osv):
    _inherit = "account.payment.term.line"
    _columns = {
        'day_tolerance': fields.integer('Days Tolerance', digits=(16,6),required=True),
        'payment_id': fields.many2one('account.payment.term', 'Payment Term', required=False, select=True),
        'compl_cash_discount':fields.boolean('Use Complete Cash Discount'),
    }
account_payment_term_line()

class account_invoice(osv.osv):
    _inherit = "account.invoice"

    def _get_amount(self, cr, uid, ids, resudial_amonut, payment_term, context=None):
        """
        This function return the Amount to paid  according to the payment term cash discount payment term lines
        """
        if context is None:
           context = {}
        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        if invoice.date_invoice:
            date1 = invoice.date_invoice
        else:
            date1 =  time.strftime('%Y-%m-%d')

        if 'date_p' in context and context['date_p']:
            date2 = context['date_p']
        else:
            date2 =  time.strftime('%Y-%m-%d')

        if date1 >= date2:
            from_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)
        else:
            from_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)

        discount = 0.0
        amount = 0.0
        tol_days = 0.0
        payment_line_found = False

        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.line_ids:
                    res = 0.0
                    dis = 0
                    for payment_line in payment_term_lines.line_ids:
                        if diff_day >= dis and  diff_day <= payment_line.days:
                            if resudial_amonut:
                                payment_line_found = True
                                tol_days = payment_line.day_tolerance
                                if payment_line.value == 'procent':
                                    res = resudial_amonut * payment_line.value_amount
                                elif payment_line.value == 'fixed':
                                    res = payment_line.value_amount
                                else:
                                    res = resudial_amonut
                            amount = res
                        dis = payment_line.days
        else:
            amount = resudial_amonut
                        
        return {'amount':amount,'tol_days':tol_days,'found':payment_line_found}

    def _get_payment(self, cr, uid, ids, resudial_amonut, payment_term, context=None):
        """
        This function return the Discount according to the payment term cash discount term
        """
        if context is None:
           context = {}

        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        if invoice.date_invoice:
            date1 = invoice.date_invoice
        else:
            date1 =  time.strftime('%Y-%m-%d')

        if 'date_p' in context and context['date_p']:
            date2 = context['date_p']
        else:
            date2 =  time.strftime('%Y-%m-%d')

        if date1 >= date2:
            from_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)
        else:
            from_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)

        tolerance = self._get_amount(cr, uid, ids, resudial_amonut, payment_term, context)
        tolerance_days = tolerance['tol_days']

        discount = 0.0
        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.cash_discount_ids:
                    res = 0.0
                    dis = 0
                    for discount_line in payment_term_lines.cash_discount_ids:
                        if diff_day >= dis and  diff_day <= discount_line.delay + tolerance_days:
                            if resudial_amonut:
                                res = resudial_amonut * discount_line.discount
                            discount = res
                        dis = discount_line.delay + tolerance_days
        return discount

    def _get_account(self, cr, uid, ids, resudial_amonut, payment_term, context=None):
        """
        This function return the Account according to the payment term cash discount term
        """
        if context is None:
           context = {}

        account_id = False

        tax_obj = self.pool.get('account.tax')
        invoice = self.browse(cr, uid, ids[0], context=context)

        if invoice.date_invoice:
            date1 = invoice.date_invoice
        else:
            date1 =  time.strftime('%Y-%m-%d')

        if 'date_p' in context and context['date_p']:
            date2 = context['date_p']
        else:
            date2 =  time.strftime('%Y-%m-%d')

        if date1 > date2:
            from_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)
        else:
            from_dt = time.mktime(time.strptime(date2,'%Y-%m-%d'))
            to_dt = time.mktime(time.strptime(date1,'%Y-%m-%d'))
            diff_day = (from_dt-to_dt)/(3600*24)

        tolerance = self._get_amount(cr, uid, ids, resudial_amonut, payment_term, context)
        tolerance_days = tolerance['tol_days']

        if payment_term:
            payment_term_lines = self.pool.get('account.payment.term').browse(cr, uid, payment_term, context=context)
            if payment_term_lines.cash_discount_ids:
                    res = 0.0
                    dis = 0.0
                    for discount_line in payment_term_lines.cash_discount_ids:
                        if diff_day >= dis and  diff_day <= discount_line.delay + tolerance_days:
                            if invoice.type in ('in_invoice', 'in_refund'):
                                account_id = discount_line.sup_discount_account_id.id
                            else:
                                account_id = discount_line.discount_account_id.id
                        dis = discount_line.delay + tolerance_days
                    return account_id

    def pay_and_reconcile(self, cr, uid, ids, pay_amount, pay_account_id, period_id, pay_journal_id, writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context=None, name=''):
        if context is None:
            context = {}
        #TODO check if we can use different period for payment and the writeoff line
        assert len(ids)==1, "Can only pay one invoice at a time"
        invoice = self.browse(cr, uid, ids[0])

        tax_obj = self.pool.get('account.tax')
        invoice_tax_obj = self.pool.get("account.invoice.tax")
        move_line_obj = self.pool.get('account.move.line')
        move_obj = self.pool.get('account.move')
        move_line_obj_memory = self.pool.get('account.move.line.memory')
        
        amount_discount = 0.0    
        if invoice.payment_term:
            # Return the discount on for the payment term
            amount_discount = self._get_payment(cr, uid, ids, pay_amount, invoice.payment_term.id, context=context)


        src_account_id = invoice.account_id.id
        # Take the seq as name for move
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        #take the choosen date
        if 'date_p' in context and context['date_p']:
            date=context['date_p']
        else:
            date=time.strftime('%Y-%m-%d')

        # Take the amount in currency and the currency of the payment
        if 'amount_currency' in context and context['amount_currency'] and 'currency_id' in context and context['currency_id']:
            amount_currency = context['amount_currency']
            currency_id = context['currency_id']
        else:
            amount_currency = False
            currency_id = False

        if invoice.type in ('in_invoice', 'in_refund'):
            ref = invoice.reference
        else:
            ref = self._convert_ref(cr, uid, invoice.number)

        discount_amount = 0.0
        tax_total_amount = 0.0

        if 'cash_amount' in context and context['cash_amount']:
            discount_amount = context['cash_amount']

        # Pay attention to the sign for both debit/credit AND amount_currency
        l1 = {
            'debit': direction * (pay_amount)>0 and direction * (pay_amount),
            'credit': direction * (pay_amount)<0 and - direction * (pay_amount),
            'account_id': src_account_id,
            'partner_id': invoice.partner_id.id,
            'ref':ref,
            'date': date,
            'currency_id':currency_id,
            'amount_currency':amount_currency and direction * amount_currency or 0.0,
        }

        ### if we change the amount paid and in the disocunt  then it used the changes discount

        if amount_discount != discount_amount and discount_amount>0.0:
            amount_discount = discount_amount

        l2 = {
            'debit': direction * (pay_amount-amount_discount)<0 and - direction * (pay_amount-amount_discount),
            'credit': direction * (pay_amount-amount_discount)>0 and direction * (pay_amount-amount_discount),
            'account_id': pay_account_id,
            'partner_id': invoice.partner_id.id,
            'ref':ref,
            'date': date,
            'currency_id':currency_id,
            'amount_currency':amount_currency and - direction * amount_currency or 0.0,
        }

        if not name:
            name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number

        l1['name'] = name
        l2['name'] = name
        lines = [(0, 0, l1), (0, 0, l2)]
    
        ### When we make payment by pay invoice wizard and if found the taxes move then directly take the entry from the wizard
        
        if 'tax_move_ids' in context and context['tax_move_ids']:
            move_line = context['tax_move_ids']
            for move_line_id in move_line:
                move_line_data = move_line_obj_memory.browse(cr, uid,move_line_id)
                l3 = {
                        'debit': move_line_data.debit,
                        'credit': move_line_data.credit,
                        'account_id': move_line_data.account_id.id,
                        'partner_id': invoice.partner_id.id,
                        'ref':ref,
                        'date': move_line_data.date,
                        'currency_id':move_line_data.currency_id.id,
                        'amount_currency':move_line_data.amount_currency and direction * move_line_data.amount_currency or 0.0,
                        'name':move_line_data.name,
                        'tax_code_id':move_line_data.tax_code_id.id,
                        'tax_amount':move_line_data.tax_amount,
                        }
                lines.append((0, 0, l3))
        else:
            if amount_discount>0.0:
                for line in invoice.invoice_line:
                    line_tax_amount = 0.0
                    for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                        line_tax_amount += tax['amount']
                    line_total = line.price_subtotal + line_tax_amount
                    invoice_res_amount = invoice.amount_total
                    line_ratio = line_total / invoice_res_amount
                    
                    if line.invoice_line_tax_id:
                        total_tax_amount = 0.0
                        for tax_id in line.invoice_line_tax_id:
                            tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                            discount_ratio = amount_discount * line_ratio
                            tax_amt = discount_ratio * tax_data.amount
                            tax_real_amt = tax_amt / (1 + tax_data.amount)
                            total_tax_amount += tax_real_amt
                            tax_invoice_id = invoice_tax_obj.search(cr,uid,[('invoice_id','=',invoice.id),('name','=',tax_data.name)])
                            
                            for tax_id in tax_invoice_id:
                                tax_value = invoice_tax_obj.browse(cr, uid, tax_id)
                                tax_account_id = tax_value.account_id.id
                                tax_code_id = tax_value.tax_code_id.id
                           
                            l3 = {
                                'debit': direction * tax_real_amt<0 and - direction * tax_real_amt,
                                'credit': direction * tax_real_amt>0 and  direction * tax_real_amt,
                                'account_id': tax_account_id,
                                'partner_id': invoice.partner_id.id,
                                'ref':ref,
                                'date': date,
                                'currency_id':currency_id,
                                'amount_currency':amount_currency and direction * amount_currency or 0.0,
                                'tax_code_id':tax_code_id,
                                'tax_amount':tax_real_amt,
                            }
                            l3['name'] = name
                            lines.append((0, 0, l3))

        ### When we make payment by pay invoice wizard and if found the discount moves in wizard discount move then directly take the entry from the wizard
        if 'discount_move_ids' in context and context['discount_move_ids']:
            move_line = context['discount_move_ids']
            for move_line_id in move_line:
                move_line_data = move_line_obj_memory.browse(cr, uid, move_line_id)
                l4 = {
                        'debit': move_line_data.debit,
                        'credit':move_line_data.credit,
                        'account_id': move_line_data.account_id.id,
                        'partner_id': invoice.partner_id.id,
                        'ref':ref,
                        'date': move_line_data.date,
                        'currency_id':move_line_data.currency_id.id,
                        'amount_currency':move_line_data.amount_currency and direction * move_line_data.amount_currency or 0.0,
                        'name':move_line_data.name,
                        'tax_code_id':move_line_data.tax_code_id.id,
                        'tax_amount':move_line_data.tax_amount,
                      }
                lines.append((0, 0, l4))
        else:
            if amount_discount>0:
                account_id = False
                if 'account_id' in context and context['account_id']:
                    account_id = context['account_id']

                for line in invoice.invoice_line:
                    line_tax_amount = 0.0
                    tax_amount = 0.0
                    discount = 0.0
                
                    for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                        line_tax_amount += tax['amount']
                    line_total = line.price_subtotal + line_tax_amount
                    invoice_res_amount = invoice.amount_total
                    line_ratio = line_total / invoice_res_amount

                    discount_ratio = amount_discount * line_ratio
                    
                    total_tax_amount = 0.0  
                    
                    for tax_id in line.invoice_line_tax_id:
                        tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                        tax_amt = discount_ratio * tax_data.amount
                        tax_real_amt = tax_amt / (1 + tax_data.amount)
                        total_tax_amount += tax_real_amt
                        
                    tax_invoice_id = invoice_tax_obj.search(cr,uid,[('invoice_id','=',invoice.id),('name','=',tax_data.name)])
                    for tax_id in tax_invoice_id:
                        tax_value = invoice_tax_obj.browse(cr, uid, tax_id)
                        tax_account_id = tax_value.account_id.id
                        base_code_id = tax_value.base_code_id.id
                        
                    l4 = {
                        'debit': direction * (discount_ratio - total_tax_amount)<0 and - direction * (discount_ratio - total_tax_amount),
                        'credit': direction * (discount_ratio - total_tax_amount)>0 and  direction * (discount_ratio - total_tax_amount),
                        'account_id': account_id,
                        'partner_id': invoice.partner_id.id,
                        'ref':ref,
                        'date': date,
                        'currency_id':currency_id,
                        'amount_currency':amount_currency and direction * amount_currency or 0.0,
                        'tax_code_id':base_code_id,
                        'tax_amount':line.price_unit,
                    }
                    l4['name'] = name
                    lines.append((0, 0, l4))
                    
        move = {'ref': ref, 'line_id': lines, 'journal_id': pay_journal_id, 'period_id': period_id, 'date': date}
        move_id = move_obj.create(cr, uid, move, context=context)

        line_ids = []
        total = 0.0
        line = self.pool.get('account.move.line')
        cr.execute('select id from account_move_line where move_id in ('+str(move_id)+','+str(invoice.move_id.id)+')')

        lines = line.browse(cr, uid, map(lambda x: x[0], cr.fetchall()) )
        for l in lines+invoice.payment_ids:
            if l.account_id.id==src_account_id:
                line_ids.append(l.id)
                total += (l.debit or 0.0) - (l.credit or 0.0)
        if (not round(total,int(config['price_accuracy']))) or writeoff_acc_id:
            self.pool.get('account.move.line').reconcile(cr, uid, line_ids, 'manual', writeoff_acc_id, writeoff_period_id, writeoff_journal_id, context)
        else:
            self.pool.get('account.move.line').reconcile_partial(cr, uid, line_ids, 'manual', context)

        # Update the stored value (fields.function), so we write to trigger recompute
        self.pool.get('account.invoice').write(cr, uid, ids, {}, context=context)
        return True

account_invoice()

class account_bank_statement_reconcile(osv.osv):
    _inherit='account.bank.statement.reconcile'
    def name_get(self, cursor, user, ids, context=None):
        res = []
        res = super(account_bank_statement_reconcile, self).name_get(cursor, user, ids, context)   
        for o in self.browse(cursor, user, ids, context=context):
            data = res[0][1]  + o.name
            res.append((o.id,data))
        return res
account_bank_statement_reconcile()    

class res_company(osv.osv):
    _inherit  = "res.company"

    _columns = {
        'expense_account_id': fields.many2one('account.account', 'Expenses',required=True),
        'revenue_account_id': fields.many2one('account.account', 'Revenues',required=True),
    }
res_company()

class res_company_wizard(osv.osv_memory):
    _name = 'res.company.wizard'
    _columns = {
        'expense_account_id': fields.many2one('account.account', 'Expenses',required=True),
        'revenue_account_id': fields.many2one('account.account', 'Revenues',required=True),
    }
    _defaults = {
        'expense_account_id': lambda *a: 2,
        'revenue_account_id': lambda *a: 3,
    }

    def set_default(self, cr, uid, ids, context=None):
        company_id =self.pool.get('res.users').browse(cr,uid,[uid],context)[0].company_id
        for o in self.browse(cr, uid, ids, context=context):
             self.pool.get('res.company').write(cr, uid, [company_id.id], {'expense_account_id':o.expense_account_id.id,'revenue_account_id':o.revenue_account_id.id})
        return {
                'view_type': 'form',
                "view_mode": 'form',
                'res_model': 'ir.actions.configuration.wizard',
                'type': 'ir.actions.act_window',
                'target': 'new',
         }

    def action_cancel(self,cr,uid,ids,conect=None):
        return {
                'view_type': 'form',
                "view_mode": 'form',
                'res_model': 'ir.actions.configuration.wizard',
                'type': 'ir.actions.act_window',
                'target':'new',
        }

res_company_wizard()

class account_bank_statement(osv.osv):
    _inherit="account.bank.statement"

    def button_confirm(self, cr, uid, ids, context={}):
        done = []
        res_currency_obj = self.pool.get('res.currency')
        res_users_obj = self.pool.get('res.users')
        account_move_obj = self.pool.get('account.move')
        account_move_line_obj = self.pool.get('account.move.line')
        cur_obj = self.pool.get('res.currency')
        account_bank_statement_line_obj = \
                self.pool.get('account.bank.statement.line')
        account_bank_reconcil = self.pool.get('account.bank.statement.reconcile')
        company_currency_id = res_users_obj.browse(cr, uid, uid,
                context=context).company_id.currency_id.id
        for st in self.browse(cr, uid, ids, context):
            if not st.state=='draft':
                continue
            if not (abs(st.balance_end - st.balance_end_real) < 0.0001):
                raise osv.except_osv(_('Error !'),
                        _('The statement balance is incorrect !\n') +
                        _('The expected balance (%.2f) is different than the computed one. (%.2f)') % (st.balance_end_real, st.balance_end))
            if (not st.journal_id.default_credit_account_id) \
                    or (not st.journal_id.default_debit_account_id):
                raise osv.except_osv(_('Configuration Error !'),
                        _('Please verify that an account is defined in the journal.'))

            for line in st.move_line_ids:
                if line.state <> 'valid':
                    raise osv.except_osv(_('Error !'),
                            _('The account entries lines are not in valid state.'))
            # for bank.statement.lines
            # In line we get reconcile_id on bank.ste.rec.
            # in bank stat.rec we get line_new_ids on bank.stat.rec.line
            for move in st.line_ids:
                context.update({'date':move.date})
                move_id = account_move_obj.create(cr, uid, {
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'date': move.date,
                }, context=context)
                account_bank_statement_line_obj.write(cr, uid, [move.id], {
                    'move_ids': [(4,move_id, False)]
                })
                if not move.amount:
                    continue
                torec = []
                if move.amount >= 0:
                    account_id = st.journal_id.default_credit_account_id.id
                else:
                    account_id = st.journal_id.default_debit_account_id.id
                acc_cur = ((move.amount<=0) and st.journal_id.default_debit_account_id) or move.account_id
                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                        company_currency_id, move.amount, context=context,
                        account=acc_cur)
#                if move.reconcile_id and move.reconcile_id.line_new_ids:
#                    for newline in move.reconcile_id.line_new_ids:
#                        amount += newline.amount

                val = {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                    'account_id': (move.account_id) and move.account_id.id,
                    'credit': ((amount>0) and amount) or 0.0,
                    'debit': ((amount<0) and -amount) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'currency_id': st.currency.id,
                }

                amount = res_currency_obj.compute(cr, uid, st.currency.id,
                        company_currency_id, move.amount, context=context,
                        account=acc_cur)
                if st.currency.id <> company_currency_id:
                    amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
                                st.currency.id, amount, context=context,
                                account=acc_cur)
                    val['amount_currency'] = -amount_cur

                if move.account_id and move.account_id.currency_id and move.account_id.currency_id.id <> company_currency_id:
                    val['currency_id'] = move.account_id.currency_id.id
                    if company_currency_id==move.account_id.currency_id.id:
                        amount_cur = move.amount
                    else:
                        amount_cur = res_currency_obj.compute(cr, uid, company_currency_id,
                                move.account_id.currency_id.id, amount, context=context,
                                account=acc_cur)
                    val['amount_currency'] = amount_cur

                torec.append(account_move_line_obj.create(cr, uid, val , context=context))
                newline_sum = 0.0
                if move.reconcile_id and move.reconcile_id.line_new_ids:
                    for newline in move.reconcile_id.line_new_ids:
                        newline_sum += newline.amount
                        account_move_line_obj.create(cr, uid, {
                            'name': newline.name or move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id': (newline.account_id) and newline.account_id.id,
                            'debit': newline.amount>0 and newline.amount or 0.0,
                            'credit': newline.amount<0 and -newline.amount or 0.0,
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,

                        }, context=context)

                amount_currency = False
                currency_id = False
                partner_acc_id = False

                if st.currency.id <> company_currency_id:
                    amount_currency = move.amount
                    expense_acc = res_users_obj.browse(cr, uid, uid, context=context).company_id.expense_account_id
                    revene_acc = res_users_obj.browse(cr, uid, uid, context=context).company_id.revenue_account_id
                    currency_id = st.currency.id

                    if move.type=='customer':
                       partner_acc_id = move.partner_id.property_account_receivable.id
                    elif move.type=='supplier':
                        partner_acc_id = move.partner_id.property_account_payable.id

                    for line in move.reconcile_id.line_ids:
                        balance = 0.0
                        move_line_data = self.pool.get('account.move.line').browse(cr, uid, line.id, context=context)
                        context.update({'amount': move_line_data.amount_currency, 'currency_id': move_line_data.currency_id.id})
                        balance = self.pool.get("account.bank.statement.reconcile").browse(cr, uid, move.reconcile_id.id, context=context).total_balance

                    if balance < 0.00:
                        account_move_line_obj.create(cr, uid, {
                            'name': move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id': expense_acc.id,
                            'credit': abs(balance),
                            'debit': 0.0,
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                            'amount_currency': amount_currency,
                            'currency_id': currency_id,
                        }, context=context)

                        account_move_line_obj.create(cr, uid, {
                            'name': move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id': partner_acc_id,
                            'credit':  0.0,
                            'debit': abs(balance),
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                            'amount_currency': amount_currency,
                            'currency_id': currency_id,
                        }, context=context)

                    elif balance > 0.00:
                        account_move_line_obj.create(cr, uid, {
                            'name': move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id':revene_acc.id,
                            'credit': abs(balance),
                            'debit':  0.0,
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                            'amount_currency': amount_currency,
                            'currency_id': currency_id,
                         }, context=context)

                        account_move_line_obj.create(cr, uid, {
                            'name': move.name,
                            'date': move.date,
                            'ref': move.ref,
                            'move_id': move_id,
                            'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                            'account_id': partner_acc_id,
                            'credit': 0.0,
                            'debit': abs(balance),
                            'statement_id': st.id,
                            'journal_id': st.journal_id.id,
                            'period_id': st.period_id.id,
                            'amount_currency': amount_currency,
                            'currency_id': currency_id,
                            }, context=context)

                account_move_line_obj.create(cr, uid, {
                    'name': move.name,
                    'date': move.date,
                    'ref': move.ref,
                    'move_id': move_id,
                    'partner_id': ((move.partner_id) and move.partner_id.id) or False,
                    'account_id': account_id,
                    'credit': (((amount-newline_sum) < 0) and -(amount-newline_sum)) or 0.0,
                    'debit': (((amount-newline_sum) > 0) and (amount-newline_sum)) or 0.0,
                    'statement_id': st.id,
                    'journal_id': st.journal_id.id,
                    'period_id': st.period_id.id,
                    'amount_currency': amount_currency,
                    'currency_id': currency_id,
                    }, context=context)

                for line in account_move_line_obj.browse(cr, uid, [x.id for x in
                        account_move_obj.browse(cr, uid, move_id,
                            context=context).line_id],
                        context=context):
                    if line.state <> 'valid':
                        raise osv.except_osv(_('Error !'),
                                _('Account move line "%s" is not valid') % line.name)

                if move.reconcile_id and move.reconcile_id.line_ids:
                    torec += map(lambda x: x.id, move.reconcile_id.line_ids)
                    #try:
                    if abs(move.reconcile_amount-move.amount)<0.0001:

                        writeoff_acc_id = False
                        #There should only be one write-off account!
                        for entry in move.reconcile_id.line_new_ids:
                            writeoff_acc_id = entry.account_id.id
                            break

                        account_move_line_obj.reconcile(cr, uid, torec, 'statement', writeoff_acc_id=writeoff_acc_id, writeoff_period_id=st.period_id.id, writeoff_journal_id=st.journal_id.id, context=context)
                    else:
                        account_move_line_obj.reconcile_partial(cr, uid, torec, 'statement', context)
                    #except:
                    #    raise osv.except_osv(_('Error !'), _('Unable to reconcile entry "%s": %.2f') % (move.name, move.amount))

                if st.journal_id.entry_posted:
                    account_move_obj.write(cr, uid, [move_id], {'state':'posted'})
            done.append(st.id)
        self.write(cr, uid, done, {'state':'confirm'}, context=context)

        return True

account_bank_statement()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:

