# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
import time
from lxml import etree
from osv import fields, osv
from tools.translate import _

class account_invoice_pay_writeoff(osv.osv_memory):
    """
    Opens the write off amount pay form.
    """
    _name = "account.invoice.pay.writeoff"
    _description = "Pay Invoice  "
    _columns = {
        'writeoff_acc_id': fields.many2one('account.account', 'Write-Off account', required=True),
        'writeoff_journal_id': fields.many2one('account.journal', 'Write-Off journal', required=True),
        'comment': fields.char('Comment', size=64, required=True),
        'analytic_id': fields.many2one('account.analytic.account','Analytic Account'),
        }
    _defaults = {
        'comment':lambda *a :'Write-Off',
        }

    def pay_and_reconcile_writeoff(self, cr, uid, ids, context=None):
        pay_invoice = self.pool.get('account.invoice.pay')
        data =  self.read(cr, uid, ids,context=context)[0]
        context.update({'write_off':data})
        self.pool.get('account.invoice.pay').pay_and_reconcile(cr, uid, ids, context=context)
        return {}

account_invoice_pay_writeoff()

class account_invoice_pay(osv.osv_memory):
    """
    Generate pay invoice wizard, user can make partial or full payment for invoice.
    """
    _name = "account.invoice.pay"
    _description = "Pay Invoice  "
    _columns = {
        'amount': fields.float('Amount paid', required=True),
        'name': fields.char('Entry Name', size=64, required=True),
        'date': fields.date('Date payment', required=True),
        'cash_residual_amount': fields.float('Residual Amount'),
        'journal_id': fields.many2one('account.journal', 'Journal/Payment Mode', required=True),
        'period_id': fields.many2one('account.period', 'Period', required=True),
        'cash_amount':fields.float('Cash Discount Amount',),
        'account_id': fields.many2one('account.account', 'Customers Cash Discount Account',),
        'cal_method_selection': fields.selection([('method_gross_methodology','Calculate Discount with Net Gross Methodology'),
                                            ('method_net_methodology','Calculate Discount with Net Methodology'),],
                                              'Method To Calculation'),
        'discount_move_ids': fields.many2many('account.move.line.memory', 'account_discount_move_rel', 'discount_account_id', 'discount_move_id', 'Account Discount Moves'),
        'tax_move_ids': fields.many2many('account.move.line.memory', 'account_tax_move_rel', 'tax_account_id', 'tax_move_id', 'Account Taxes Moves'),
        }
    
    def _get_period(self, cr, uid, context=None):
        ids = self.pool.get('account.period').find(cr, uid, context=context)
        period_id = False
        if len(ids):
            period_id = ids[0]
        return period_id

    def _get_amount(self, cr, uid, context=None):
        obj_inv = self.pool.get('account.invoice')
        tax_obj = self.pool.get('account.tax')
        invoice =  obj_inv.browse(cr, uid, context['id'], context=context)
        total_tax_amount = 0.0 
        total_amount = 0.0
        amount = obj_inv._get_amount(cr, uid, [context['id']] ,invoice.residual, invoice.payment_term.id, context=context)
        context.update({'found':amount['found']})
        amount = amount['amount']
        discount = obj_inv._get_payment(cr, uid, [context['id']] ,amount, invoice.payment_term.id, context=context)
        residual_amount = amount - discount
        return residual_amount
    
    def _get_discount(self, cr, uid, context=None):
        obj_inv = self.pool.get('account.invoice')
        tax_obj = self.pool.get('account.tax')
        invoice =  obj_inv.browse(cr, uid, context['id'], context=context)
        amount = obj_inv._get_amount(cr, uid, [context['id']] ,invoice.residual, invoice.payment_term.id, context=context)
        amount = amount['amount']
        discount = obj_inv._get_payment(cr, uid, [context['id']] , amount, invoice.payment_term.id, context=context)
        return discount
    
    def _get_account(self, cr, uid, context=None):
        """
        This function return the Account according to the payment term cash term and map the account with fiscal position
        """
        obj_inv = self.pool.get('account.invoice')
        invoice =  obj_inv.browse(cr, uid, context['id'], context=context)
        account = obj_inv._get_account(cr, uid, [context['id']] , invoice.residual, invoice.payment_term.id, context=context)
        fpos = invoice.fiscal_position or False
        if account:
            account = self.pool.get('account.fiscal.position').map_account(cr, uid, fpos, account)
        return account
    
    def _get_discount_move(self, cr, uid, context=None):
        
        cur_obj = self.pool.get('res.currency')
        invoice_obj = self.pool.get('account.invoice')

        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context=context)
        tax_obj = self.pool.get('account.tax')
        invoice_tax_obj = self.pool.get("account.invoice.tax")
        move_line_obj_memory = self.pool.get('account.move.line.memory')
        
        #### to get direction
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        date=time.strftime('%Y-%m-%d')
        
        #### to get name
        name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number
        ##### Entry in the discount account moves and entry in the tax account moves
        
        amount = 0.0
        discount = 0.0
        discount_line_ids = []
        discount_account_id = False
        base_code_id = False
        
        discount = self._get_discount(cr, uid, context=context)
        discount_account_id = self._get_account(cr, uid, context=context)
        if discount>0.0:
            for line in invoice.invoice_line:
                total_tax_amount = 0.0
                ########
                line_tax_amount = 0.0
                for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                    line_tax_amount += tax['amount']
                line_total = line.price_subtotal + line_tax_amount
                invoice_res_amount = invoice.amount_total
                line_ratio = line_total / invoice_res_amount
                #####################################################
                discount_pay = discount * line_ratio
                
                if line.invoice_line_tax_id:
                    for tax_id in line.invoice_line_tax_id:
                        tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                        tax_amt = discount_pay * tax_data.amount
                        tax_real_amt = tax_amt / (1 + tax_data.amount)
                        total_tax_amount += tax_real_amt
                        tax_invoice_id = invoice_tax_obj.search(cr,uid,[('invoice_id','=',invoice.id),('name','=',tax_data.name)])
                        for tax_id in tax_invoice_id:
                            tax_value = invoice_tax_obj.browse(cr, uid, tax_id)
                            base_code_id = tax_value.base_code_id.id
                            
                if invoice.company_id.currency_id.id<>invoice.currency_id.id:
                    discount_amount = discount - tax_real_amt
                    amount = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, discount_amount, context=context)
                
                l4 = {
                    'debit': direction * (discount_pay - total_tax_amount)<0 and - direction * (discount_pay - total_tax_amount),
                    'credit': direction * (discount_pay - total_tax_amount)>0 and  direction * (discount_pay - total_tax_amount),
                    'account_id': discount_account_id,
                    'date': date,
                    'name':name,
                    'currency_id':invoice.currency_id.id or False,
                    'journal_id':invoice.journal_id.id,
                    'tax_code_id':base_code_id,
                    'tax_amount':-(discount_pay - total_tax_amount),
                    'amount_currency':amount and direction * amount or 0.0,
                    'analytic_account_id':line.account_analytic_id.id or False,
                    }
                
                discount_line_id = move_line_obj_memory.create(cr, uid, l4, context=context)
                discount_line_ids.append(l4)
        return discount_line_ids
    
    def _get_taxes_move(self, cr, uid, context=None):
        cur_obj = self.pool.get('res.currency')
        invoice_obj = self.pool.get('account.invoice')
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context=context)
        tax_obj = self.pool.get('account.tax')
        invoice_tax_obj = self.pool.get("account.invoice.tax")
        move_line_obj_memory = self.pool.get('account.move.line.memory')
        
        #### to get direction
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        date=time.strftime('%Y-%m-%d')
        
        #### to get name
        name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number
        ##### Entry in the discount account moves and entry in the tax account moves
        amount = 0.0
        discount = 0.0
        line_ids = []

        tax_code_id = False
        tax_account_id = False
        discount = self._get_discount(cr, uid, context=context)
        if discount>0.0:
            for line in invoice.invoice_line:
                line_tax_amount = 0.0
                total_tax_amount = 0.0
                ############# Ratio#####
                for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                    line_tax_amount += tax['amount']
                line_total = line.price_subtotal + line_tax_amount
                invoice_res_amount = invoice.amount_total
                line_ratio = line_total / invoice_res_amount
                #############
                ### discount for the invoice line
                discount_pay = discount * line_ratio
                                
                if line.invoice_line_tax_id:
                    for tax_id in line.invoice_line_tax_id:
                        tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                        tax_amt = discount_pay * tax_data.amount
                        tax_real_amt = tax_amt / (1 + tax_data.amount)
                        total_tax_amount += tax_real_amt
                        tax_invoice_id = invoice_tax_obj.search(cr,uid,[('invoice_id','=',invoice.id),('name','=',tax_data.name)])
                        for tax_id in tax_invoice_id:
                            tax_value = invoice_tax_obj.browse(cr, uid, tax_id)
                            tax_account_id = tax_value.account_id.id
                            tax_code_id = tax_value.tax_code_id.id

                        if invoice.company_id.currency_id.id<>invoice.currency_id.id:
                            amount = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, tax_amount, context=context)
                                
                        l3 = {
                            'debit': direction * tax_real_amt<0 and - direction * tax_real_amt,
                            'credit': direction * tax_real_amt>0 and  direction * tax_real_amt,
                            'account_id': tax_account_id,
                            'date': date,
                            'journal_id':invoice.journal_id.id or False,
                            'name':name,
                            'currency_id':invoice.currency_id.id or False,
                            'tax_code_id':tax_code_id,
                            'tax_amount':-tax_real_amt,
                            'amount_currency':amount and direction * amount or 0.0,
                            'analytic_account_id':line.account_analytic_id.id or False,
                        }
                        line_id = move_line_obj_memory.create(cr, uid, l3, context=context)
                        line_ids.append(l3)
        return line_ids
         
    _defaults = {
        'date': lambda *a: time.strftime('%Y-%m-%d'),
        'period_id': _get_period,
        'amount': _get_amount,
        'cash_amount':_get_discount,
        'account_id':_get_account,
        'cash_residual_amount': lambda *a:0.0,
        'cal_method_selection':lambda *a :'method_gross_methodology',
        'discount_move_ids':_get_discount_move,
        'tax_move_ids':_get_taxes_move,
        }
    
    def on_change_ammount(self, cr, uid, ids, amount, cash_amount, context=None):
        """
        Function return the Discount according to the Amount paid and Payment Term Cash Discount
        """
        res = {}
        obj_inv = self.pool.get('account.invoice')
        invoice =  obj_inv.browse(cr, uid, context['id'], context=context)
        cal_amount = obj_inv._get_amount(cr, uid, [context['id']] ,invoice.residual, invoice.payment_term.id, context=context)
        
        old_cal_amount = cal_amount['amount']
        old_amount = old_cal_amount 
        diff_amount = round(old_amount -  (amount + cash_amount),2)
        return {'value' : {'cash_residual_amount':diff_amount}}
    
    def on_change_cash_discount_amount(self, cr, uid, ids, discount_amount, amount, context=None):
        ###  Return discount amount
        res = {}
        obj_inv = self.pool.get('account.invoice')
        invoice =  obj_inv.browse(cr, uid, context['id'], context=context)
        cal_amount = obj_inv._get_amount(cr, uid, [context['id']] ,invoice.residual, invoice.payment_term.id, context=context)
        old_cal_amount = cal_amount['amount']
        discount = obj_inv._get_payment(cr, uid, [context['id']] , amount, invoice.payment_term.id, context=context)
        old_amount = old_cal_amount

        diff_amount = 0.0
        diff_amount = old_amount -  (amount + discount_amount)
        
        return {'value' : {'cash_amount':discount_amount,'cash_residual_amount':diff_amount}}
    
    def _calculation(self, cr, uid, ids, context=None):
        cur_obj = self.pool.get('res.currency')
        invoice_obj = self.pool.get('account.invoice')
        data =  self.read(cr, uid, ids,context=context)[0]
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context=context)
        tax_obj = self.pool.get('account.tax')
        invoice_tax_obj = self.pool.get("account.invoice.tax")
        move_line_obj_memory = self.pool.get('account.move.line.memory')
        
        #### to get direction
        types = {'out_invoice': -1, 'in_invoice': 1, 'out_refund': 1, 'in_refund': -1}
        direction = types[invoice.type]
        date=time.strftime('%Y-%m-%d')
        
        #### to get name
        name = invoice.invoice_line and invoice.invoice_line[0].name or invoice.number
        ##### Entry in the discount account moves and entry in the tax account moves

        if data.get('cash_amount', 0.0) <= 0.0:
            return True
        
        tax_total_amount = 0.0
        amount = 0.0
        discount = 0.0
        line_ids = []
        discount_line_ids = []

        discount_account_id = False
        tax_code_id = False
        base_code_id = False
        tax_account_id = False
        
        for line in invoice.invoice_line:
            tax_amount = 0.0
            line_tax_amount = 0.0
            total_tax_amount = 0.0
            #####get the ratio of the line in the total invoice amount 
            for tax in tax_obj.compute(cr, uid, line.invoice_line_tax_id, line.price_subtotal/line.quantity, line.quantity, invoice.address_invoice_id.id, line.product_id, invoice.partner_id):    
                line_tax_amount = tax['amount']
            line_total = line.price_subtotal + line_tax_amount
            invoice_res_amount = invoice.amount_total
            line_ratio = line_total / invoice_res_amount
            ###########################
            discount_pay = data['cash_amount'] * line_ratio
            if line.invoice_line_tax_id:
                for tax_id in line.invoice_line_tax_id:
                    tax_data = tax_obj.browse(cr, uid, tax_id.id, context=context)
                    tax_amt = discount_pay * tax_data.amount
                    tax_real_amt = tax_amt / (1 + tax_data.amount)
                    total_tax_amount += tax_real_amt

                    tax_invoice_id = invoice_tax_obj.search(cr,uid,[('invoice_id','=',invoice.id),('name','=',tax_data.name)])
                    for tax_id in tax_invoice_id:
                        tax_value = invoice_tax_obj.browse(cr, uid, tax_id)
                        tax_account_id = tax_value.account_id.id
                        tax_code_id = tax_value.tax_code_id.id
                        base_code_id = tax_value.base_code_id.id
                    
                    if invoice.company_id.currency_id.id<>invoice.currency_id.id:
                        amount = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, tax_amount, context=context)
                        
                    l3 = {
                        'debit': direction * tax_real_amt<0 and - direction * tax_real_amt,
                        'credit': direction * tax_real_amt>0 and  direction * tax_real_amt,
                        'account_id': tax_account_id,
                        'date': date,
                        'journal_id':data.get('journal_id',False) ,
                        'name':name,
                        'currency_id':invoice.currency_id.id or False,
                        'tax_code_id':tax_code_id,
                        'tax_amount':-tax_real_amt,
                        'amount_currency':amount and direction * amount or 0.0,
                        'analytic_account_id':line.account_analytic_id.id or False,
                    }
                    line_id = move_line_obj_memory.create(cr, uid, l3, context=context)
                    line_ids.append(line_id)
        
         ### if calculation on the Net Methology then in discount field it will take product income account or product category income account 
            if data['cal_method_selection'] == 'method_net_methodology':
                if invoice.type == 'in_invoice':
                    if line.product_id.property_account_income:
                        discount_account_id = line.product_id.property_account_income.id
                    else:
                        discount_account_id = line.product_id.categ_id.property_account_income_categ.id
                else:
                    if line.product_id.property_account_income:
                        discount_account_id = line.product_id.property_account_expense.id
                    else:
                        discount_account_id = line.product_id.categ_id.property_account_expense_categ.id
                            
            else:
                 discount_account_id = data['account_id']
            
            if invoice.company_id.currency_id.id<>invoice.currency_id.id:
                discount_amount = discount_pay - tax_amount
                amount = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, discount_amount, context=context)

            l4 = {
                'debit': direction * (discount_pay - total_tax_amount)<0 and - direction * (discount_pay - total_tax_amount),
                'credit': direction * (discount_pay - total_tax_amount)>0 and  direction * (discount_pay - total_tax_amount),
                'account_id': discount_account_id,
                'date': date,
                'name':name,
                'currency_id':invoice.currency_id.id or False,
                'journal_id':data.get('journal_id',False),
                'tax_code_id':base_code_id,
                'tax_amount':-(discount_pay - total_tax_amount),
                'amount_currency':amount and direction * amount or 0.0,
                'analytic_account_id':line.account_analytic_id.id or False,
                }
            
            discount_line_id = move_line_obj_memory.create(cr, uid, l4, context=context)
            discount_line_ids.append(discount_line_id)

        self.write(cr, uid, ids,
               {'tax_move_ids':[(6,0,line_ids)],
                'discount_move_ids' : [(6, 0, discount_line_ids)],
               }, context=context)

        return True
    
    def default_get(self, cr, uid, fields, context=None):
      res = super(account_invoice_pay, self).default_get(cr, uid, fields, context=context)
      invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context=context)
      if invoice.state in ['draft', 'proforma2', 'cancel','paid']:
          raise osv.except_osv(_('Error !'), _('Can not pay draft/proforma/cancel/Done invoice.'))
      return res
     
    def _message(self, cr, uid, ids, context=None):
        mod_obj = self.pool.get('ir.model.data')
        data =  self.read(cr, uid, ids,context=context)[0]
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context)
        
        if data.get('discount_move_ids',False):
            context.update({'discount_move_ids':data['discount_move_ids']})
        if data.get('tax_move_ids',False):
            context.update({'tax_move_ids':data['tax_move_ids']})
        if data.get('amount',False):
            context.update({'amount':data['amount']})
        if data.get('cash_amount',False):
            context.update({'cash_amount':data['cash_amount']})
            
        total_data_amount = 0.00
        total_invoice_amount = 0.00
        total_data_amount = data['amount'] + data['cash_amount']
        total_invoice_amount = invoice.residual
        
                      
        model_data_ids = mod_obj.search(cr, uid,[('model','=','ir.ui.view'),('name','=','view_account_message')], context=context)
        resource_id = mod_obj.read(cr, uid, model_data_ids, fields=['res_id'], context=context)[0]['res_id']
        if self.pool.get('res.currency').is_zero(cr, uid, invoice.company_id.currency_id,(total_invoice_amount - total_data_amount)):
            self.pool.get('account.invoice.pay').pay_and_reconcile(cr, uid, ids, context=context)
            return {}
        else:
            return {
                'name': _('Message'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'account.message',
                'views': [(resource_id,'form')],
                'type': 'ir.actions.act_window',
                'target': 'new',
            }
                  
    
    def _message_partial(self, cr, uid, ids, context=None):
        mod_obj = self.pool.get('ir.model.data')
        data =  self.read(cr, uid, ids,context=context)[0]
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context)

        total_data_amount = 0.0
        total_invoice_amount = 0.0
        
        if data.get('discount_move_ids',False):
            context.update({'discount_move_ids':data['discount_move_ids']})
        if data.get('tax_move_ids',False):
            context.update({'tax_move_ids':data['tax_move_ids']})

        if data.get('amount',False):
            context.update({'amount':data['amount']})
        if data.get('cash_amount',False):
            context.update({'cash_amount':data['cash_amount']})
        
        total_data_amount = data['amount'] + data['cash_amount']
        total_invoice_amount = invoice.residual
        
        if self.pool.get('res.currency').is_zero(cr, uid, invoice.company_id.currency_id,(total_invoice_amount - total_data_amount)):
            model_data_ids = mod_obj.search(cr, uid,[('model','=','ir.ui.view'),('name','=','view_account_partial_message')], context=context)
            resource_id = mod_obj.read(cr, uid, model_data_ids, fields=['res_id'], context=context)[0]['res_id']
            return {
                'name': _('Message'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'account.partial.message',
                'views': [(resource_id,'form')],
                'type': 'ir.actions.act_window',
                'target': 'new',   
            }
        else:
            self.pool.get('account.invoice.pay').pay_and_reconcile(cr, uid, ids, context=context)
            return {}      
            
    def wo_check(self, cr, uid, ids, context=None):
        cur_obj = self.pool.get('res.currency')
        mod_obj = self.pool.get('ir.model.data')
        obj_inv = self.pool.get('account.invoice')
        
        if context is None:
            context = {}
        data =  self.read(cr, uid, ids,context=context)[0]
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context)
        journal = self.pool.get('account.journal').browse(cr, uid, data['journal_id'], context)
       # Here we need that:
        #    The invoice total amount in company's currency <> paid amount in company currency
        #    (according to the correct day rate, invoicing rate and payment rate are may be different)
        #    => Ask to a write-off of the difference. This could happen even if both amount are equal,
        #    because if the currency rate
        # Get the amount in company currency for the invoice (according to move lines)
        inv_amount_company_currency = 0
        for aml in invoice.move_id.line_id:
            if aml.account_id.id == invoice.account_id.id or aml.account_id.type in ('receivable', 'payable'):
                inv_amount_company_currency += aml.debit
                inv_amount_company_currency -= aml.credit
        inv_amount_company_currency = abs(inv_amount_company_currency)
        # Get the current amount paid in company currency
        if journal.currency and invoice.company_id.currency_id.id<>journal.currency.id:
            ctx = {'date':data['date']}
            amount_paid = cur_obj.compute(cr, uid, journal.currency.id, invoice.company_id.currency_id.id, data['amount'], round=True, context=ctx)
        else:
            amount_paid = data['amount'] + data['cash_amount']
            
        # Get the old payment if there are some
        if invoice.payment_ids:
            debit=credit=0.0
            for payment in invoice.payment_ids:
                debit+=payment.debit
                credit+=payment.credit
            amount_paid+=abs(debit-credit)
        
        if data['cash_residual_amount'] == 0.0 or  self.pool.get('res.currency').is_zero(cr, uid, invoice.company_id.currency_id,
                (amount_paid - inv_amount_company_currency)):
            return self.pay_and_reconcile(cr, uid, ids, context=context)
        else:
            model_data_ids = mod_obj.search(cr, uid,[('model','=','ir.ui.view'),('name','=','view_account_invoice_pay_writeoff')], context=context)
            resource_id = mod_obj.read(cr, uid, model_data_ids, fields=['res_id'], context=context)[0]['res_id']
            return {
                'name': _('Information addendum'),
                'context': context,
                'view_type': 'form',
                'view_mode': 'form',
                'res_model': 'account.invoice.pay.writeoff',
                'views': [(resource_id,'form')],
                'type': 'ir.actions.act_window',
                'target': 'new',
            }

    def pay_and_reconcile(self, cr, uid, ids, context=None):
        cur_obj = self.pool.get('res.currency')
        if context is None:
            context = {}

        data =  self.read(cr, uid, ids,context=context)[0]
        writeoff_account_id = False
        writeoff_journal_id = False
        comment = False
        
        if 'write_off' in context and context['write_off'] :
            writeoff_account_id = context['write_off']['writeoff_acc_id']
            writeoff_journal_id = context['write_off']['writeoff_journal_id']
            comment = context['write_off']['comment']
       
        if context.get('amount',False):
            amount = context['amount'] +  context.get('cash_amount',0.0)
        else:
            amount = data['amount'] + data['cash_amount']
        
        invoice = self.pool.get('account.invoice').browse(cr, uid, context['id'], context=context)
        journal = self.pool.get('account.journal').browse(cr, uid, data['journal_id'], context=context)
        # Compute the amount in company's currency, with the journal currency (which is equal to payment currency)
        # when it is needed :  If payment currency (according to selected journal.currency) is <> from company currency
        if journal.currency and invoice.company_id.currency_id.id<>journal.currency.id:
            ctx = {'date':data['date']}
            amount = cur_obj.compute(cr, uid, journal.currency.id, invoice.company_id.currency_id.id, amount, context=ctx)
            currency_id = journal.currency.id
            # Put the paid amount in currency, and the currency, in the context if currency is different from company's currency
            if context.get('amount',False):
                context.update({'amount_currency':context['amount'] +  context.get('cash_amount',0.0),'currency_id':currency_id})
            else:    
                context.update({'amount_currency':data['amount'] + data['cash_amount'],'currency_id':currency_id})


        if invoice.company_id.currency_id.id<>invoice.currency_id.id:
            ctx = {'date':data['date']}
            amount = cur_obj.compute(cr, uid, invoice.currency_id.id, invoice.company_id.currency_id.id, amount, context=ctx)
            currency_id = invoice.currency_id.id
            # Put the paid amount in currency, and the currency, in the context if currency is different from company's currency
            if context.get('amount',False):
                context.update({'amount_currency':context['amount'] +  context.get('cash_amount',0.0),'currency_id':currency_id})
            else:    
                context.update({'amount_currency':data['amount'] + data['cash_amount'],'currency_id':currency_id})

        context.update({'account_id':data['account_id'],'date_p':data['date'],'comment':comment or False})        

        if context.get('discount_move_ids',False):
            context.update({'discount_move_ids':context['discount_move_ids']})
        else:
            context.update({'discount_move_ids':data['discount_move_ids']})    

        if context.get('tax_move_ids',False):
            context.update({'tax_move_ids':context['tax_move_ids']})
        else:
            context.update({'tax_move_ids':data['tax_move_ids']})
        
        if context.get('cash_amount',False):
            context.update({'cash_amount':context['cash_amount']})        
        else:
            context.update({'cash_amount':data['cash_amount']})
            
        acc_id = journal.default_credit_account_id and journal.default_credit_account_id.id
        if not acc_id:
            raise osv.except_osv(_('Error !'), _('Your journal must have a default credit and debit account.'))

        self.pool.get('account.invoice').pay_and_reconcile(cr, uid, [context['id']],
                amount, acc_id, data['period_id'], data['journal_id'], writeoff_account_id,
                data['period_id'], writeoff_journal_id, context, data['name'])
        return {}
    
    def fields_view_get(self, cr, uid, view_id=None, view_type='form', context=None, toolbar=False):
        res = super(account_invoice_pay,self).fields_view_get(cr, uid, view_id, view_type, context, toolbar)
        fields=res.get('fields',{})
        word = "Supplier Cash Discount Account"
        type = context.get('type',False)
        eview = etree.fromstring(res['arch'])
        res['arch'] = etree.tostring(eview)
        if type == 'in_invoice':
            for f in res['fields']:
                if 'Customers Cash Discount Account' in res['fields'][f]['string']:
                    res['fields'][f]['string'] = res['fields'][f]['string'].replace('Customers Cash Discount Account',word)
        return res
    
account_invoice_pay()

class account_move_line_memory(osv.osv_memory):
    _name= "account.move.line.memory"
    _description = "Account move line memory "
    _columns = {
        'date': fields.date('Effective Date', required=True),
        'name': fields.char('Name', size=64, required=True),
        'account_id': fields.many2one('account.account', 'Account',),
        'journal_id': fields.many2one('account.journal', 'Journal', required=True, select=1),
        'debit': fields.float('Debit', digits=(16,2)),
        'credit': fields.float('Credit', digits=(16,2)),
        'tax_code_id': fields.many2one('account.tax.code', 'Tax Account', help="The Account can either be a base tax code or tax code account."),
        'tax_amount': fields.float('Tax/Base Amount', digits=(16,2), select=True),
        'currency_id': fields.many2one('res.currency', 'Currency', help="The optional other currency if it is a multi-currency entry."),
        'amount_currency': fields.float('Amount Currency', help="The amount expressed in an optional other currency if it is a multi-currency entry."),
        'analytic_account_id' : fields.many2one('account.analytic.account', 'Analytic Account'),
        }
account_move_line_memory()

class account_message(osv.osv_memory):
    _name = "account.message"
    def _check(self, cr, uid, ids, context=None):
        data =  self.read(cr, uid, ids,context=context)[0]
        result = self.pool.get('account.invoice.pay').wo_check(cr, uid, ids, context=context)
        return result

account_message()

class account_partial_message(osv.osv_memory):
    _name = "account.partial.message"
    def _check_partial(self, cr, uid, ids, context=None):
        data =  self.read(cr, uid, ids,context=context)[0]
        result = self.pool.get('account.invoice.pay').pay_and_reconcile(cr, uid, ids, context=context)
        return result

account_partial_message()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4: