# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (c) 2009 Zikzakmedia S.L. (http://zikzakmedia.com) All Rights Reserved.
#                       Jordi Esteve <jesteve@zikzakmedia.com>
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields
from tools.sql import drop_view_if_exists

class stock_picking(osv.osv):
    _name='stock.picking'
    _inherit='stock.picking'
    
    def _compute_quantity(self, cr, uid, ids, fieldnames, args, context=None):
        res = {}
        for obj in self.browse(cr, uid, ids, contex=context):
            total_quantity = sum(move_id.product_qty for move_id in obj.move_lines)
            scanned_quantity = sum(scan.scan_quantity for scan in obj.scaned_ids)
            res[obj.id] = {
                'total_quantity' : total_quantity,
                'scanned_quantity' : scanned_quantity,
                'to_be_scanned_quantity' : total_quantity - scanned_quantity,
            }

        return res

    
    _columns = {
        'ean' : fields.char("EAN", size=64,),
        'total_quantity':fields.function(_compute_quantity, method=True, string='Total Quantity', multi='quantity'),
        'scanned_quantity': fields.function(_compute_quantity, method=True, string='Scanned Quantity', multi='quantity'),
        'to_be_scanned_quantity':fields.function(_compute_quantity, method=True, string='To Be Scanned Quantity', multi='quantity'),
        'tobe_scan_ids':fields.one2many('tobe.scanned.stock', 'tobe_picking_id', 'To Be'),
        'scaned_ids':fields.one2many('scanned.stock', 'scanned_picking_id', 'Scanned Picking',),
    }
    _defaults = {
      'scanned_quantity' : lambda *a:0.0,
    }

stock_picking()

class stock_move(osv.osv):
    _inherit = "stock.move"
    _columns = {
       'scaned_qty':fields.float("Scanned Qty"),
    }
    
    _defaults = {
      'scaned_qty' : lambda *a:0.0,
    }
stock_move()

class tobe_scanned_stock(osv.osv):
    _name = "tobe.scanned.stock"
    _auto = False
    _columns = {
        'tobe_picking_id': fields.many2one('stock.picking', 'Stock Picking',),
        'product_name':fields.many2one('product.product','Product Name', size=64),
        'lot_number':fields.many2one('stock.production.lot','Lot Number', size=64),
        'quantity':fields.float("Quantity"),
        'scan_quantity':fields.float("Scan Quantity"),
    }
    
    def init(self, cr):
        drop_view_if_exists(cr, 'tobe_scanned_stock')
        cr.execute("""
            CREATE OR REPLACE VIEW tobe_scanned_stock AS (
                SELECT
                    l.id AS id,
                    l.picking_id AS tobe_picking_id,
                    sum(l.scaned_qty) AS scan_quantity,
                    sum(l.product_qty) AS quantity,
                    l.prodlot_id AS lot_number,
                    l.product_id AS product_name
                FROM
                    stock_move l
                WHERE
                    l.scaned_qty != l.product_qty
                GROUP BY
                    l.product_id,l.prodlot_id,l.picking_id, l.id
            )
        """)
tobe_scanned_stock()

class scanned_stock(osv.osv):
    _name = "scanned.stock"
    _auto = False
    _columns = {
        'scanned_picking_id': fields.many2one('stock.picking', 'Stock Picking',),        
        'product_name':fields.many2one('product.product','Product Name', size=64),
        'lot_number':fields.many2one('stock.production.lot','Lot Number', size=64),
        'quantity':fields.float("Quantity"),
        'scan_quantity':fields.float("Scan Quantity"),
    }
    
    def init(self, cr):
        drop_view_if_exists(cr, 'scanned_stock')
        cr.execute("""
            CREATE OR REPLACE VIEW scanned_stock AS (
                SELECT
                    l.id AS id,
                    l.picking_id AS scanned_picking_id,
                    sum(l.scaned_qty) AS scan_quantity,
                    sum(l.product_qty) AS quantity,
                    l.prodlot_id AS lot_number,
                    l.product_id AS product_name
                FROM
                    stock_move l
                WHERE
                    l.scaned_qty = l.product_qty
                GROUP BY
                    l.product_id,l.prodlot_id,l.picking_id, l.id
            )
        """)
scanned_stock()


# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
