# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import time
from tools.translate import _
from osv import osv

scan_product_ean_form = """<?xml version="1.0"?>
<form string="Scan Product">
    <field name="ean"/>
</form>"""

scan_product_ean_fields = {
    'ean': {'string': 'EAN number', 'type': 'char','required': True, 'size':32},
}

def _scan_product_ean(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    stock_move_obj = pool.get("stock.move")
    picking_id = data['ids']
    form_ean = data['form']['ean']
    stock_move_ids = stock_move_obj.search(cr, uid, [('picking_id','=',picking_id[0])])

    for move_id in stock_move_ids:
        move_data = stock_move_obj.browse(cr, uid,move_id)
        if move_data.product_qty == move_data.scaned_qty:
            continue
        for ean_id in move_data.product_id.packaging:
            ean_data = pool.get("product.packaging").browse(cr, uid, ean_id.id)
            product_ean = ean_data.ean
            if product_ean == form_ean:
                if move_data.product_id.track_production:
                    return 'scan_product_number'
                else:
                    stock_move_obj.write(cr, uid, [move_id],{'scaned_qty':move_data.scaned_qty+1}, context=context)
                    return 'init'
    else:
        raise osv.except_osv(_('Warning!'),
                             _('Product does not belongs to this picking or already scanned.'))
    return {}

scan_product_lot_form = """<?xml version="1.0"?>
<form string="Scan Product Lot Number">
    <field name="lot_number_id"/>
</form>"""

scan_product_lot_fields = {
    'lot_number_id': {'string':'Lot Number', 'type': 'many2one', 'relation':'stock.production.lot', 'required':True},
}

def _scan_product_lot_number(self, cr, uid, data, context):
    pool = pooler.get_pool(cr.dbname)
    move_obj = pool.get("stock.move")
    stock_pciking_obj = pool.get("stock.picking")
    picking_ids = data['ids']
    form_lot_number = data['form']['lot_number_id']
    for picking_id in picking_ids:
        pick_data = stock_pciking_obj.browse(cr, uid,picking_id)
        for pick in pick_data.tobe_scan_ids:
            stock_lot_ids = move_obj.search(cr, uid, [('picking_id','=',picking_ids[0]),('prodlot_id','=',form_lot_number)])
            call_product_scan = 1
            if stock_lot_ids:
                for stock_move_obj in move_obj.browse(cr, uid, stock_lot_ids, context):
                    if stock_move_obj.product_qty == stock_move_obj.scaned_qty:
                        call_product_scan = 1
                        continue
                    move_obj.write(cr, uid, [stock_move_obj.id],{'scaned_qty':stock_move_obj.scaned_qty+1}, context=context)
                    call_product_scan = 2
                    if stock_move_obj.product_qty >= stock_move_obj.scaned_qty+1:
                        call_product_scan = 1
                    break
                if call_product_scan == 1:
                    return 'init'
                return 'scan_product_number'
            else:
                raise osv.except_osv(_('Warning!'),
                                     _('Product lot does not belongs to this picking'))
    return 'end'

def _get_defaults(self, cr, uid, data, context):
    data['form'].update({'ean' : '', 'lot_number_id' : False})
    return data['form']

class select_picking(wizard.interface):
    states = {
        'init' : {
            'actions' : [_get_defaults],
            'result' : {'type': 'form',
                    'arch' : scan_product_ean_form,
                    'fields' : scan_product_ean_fields,
                    'state' : [('end', 'Cancel' ,'gtk-cancel'),('scan_product_ean', 'Ok', 'gtk-ok', True) ]}
        },
        'scan_product_ean' : {
            'actions' : [],
            'result' : {'type' : 'choice',
                    'next_state' : _scan_product_ean}
            },
        'scan_product_number': {
            'actions': [],
            'result': {'type': 'form', 'arch':scan_product_lot_form, 'fields':scan_product_lot_fields, 'state':[('end','Cancel', 'gtk-cancel'),('scan_product_lot_number','Ok', 'gtk-ok', True)]}
        },
         'scan_product_lot_number': {
            'actions': [],
            'result' : {'type' : 'choice',
                    'next_state' : _scan_product_lot_number}
        },
         'end' : {
            'actions' : [],
            'result': {'type': 'state', 'state': 'end'}
            },
    }
select_picking("scan.product")

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
