# -*- encoding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2009 Tiny SPRL (<http://tiny.be>). All Rights Reserved
#    $Id$
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU General Public License as published by
#    the Free Software Foundation, either version 3 of the License, or
#    (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU General Public License for more details.
#
#    You should have received a copy of the GNU General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv

class sale_order_line(osv.osv):
    _inherit = "sale.order.line"
    _columns = {
        'sale_price_unit_tax': fields.float('Sale price W/O taxes'),
    }
    def product_id_change(self, cr, uid, ids, pricelist, product, qty=0,
                          uom=False, qty_uos=0, uos=False, name='', partner_id=False,
                          lang=False, update_tax=True, date_order=False, packaging=False,
                          fiscal_position=False, flag=False):

        result = super(sale_order_line, self).product_id_change(cr, uid, ids, pricelist, product, qty,
                                                                uom, qty_uos, uos, name, partner_id, lang,
                                                                update_tax, date_order, packaging, fiscal_position, flag)
        if product:
            context = {'lang': lang, 'partner_id': partner_id, 'qty': qty}
            product_obj = self.pool.get('product.product').browse(cr, uid, product, context=context)
            result['value']['sale_price_unit_tax'] = product_obj.product_tmpl_id.price_unit_tax
        return result

sale_order_line()

class sale_order(osv.osv):
    _name = "sale.order"
    _inherit = "sale.order"

    def onchange_shop_id(self, cr, uid, ids, shop_id):
        res = super(sale_order, self).onchange_shop_id(cr, uid, id, shop_id)
        if 'value' in res and shop_id:
            res['value']['price_type'] = self.pool.get('sale.shop').browse(cr, uid , shop_id).price_type
        return res

sale_order()

class sale_shop(osv.osv):
    _name = 'sale.shop'
    _inherit = 'sale.shop'
    _columns = {
          'price_type': fields.selection([
            ('tax_included','Tax included'),
            ('tax_excluded','Tax excluded')
        ], 'Price method', required=True),
    }
    _defaults = {
        'price_type': lambda *a: 'tax_included',
    }

sale_shop()

class sale_shop_price_method_policy(osv.osv_memory):
    _name = 'sale.shop.price.method'
    _columns = {
        'name': fields.char('Name', size=64),
        'price_type': fields.selection([
            ('tax_included','Tax included'),
            ('tax_excluded','Tax excluded')
        ], 'Price method', required=True),
    }
    _defaults = {
        'price_type': lambda *a: 'tax_included',
    }

    def set_default(self, cr, uid, ids, context=None):
        ir_values_obj = self.pool.get('ir.values')
        for obj in self.browse(cr, uid, ids, context=context):
            ir_values_obj.set(cr, uid, 'default', False, 'price_type', ['sale.shop'], obj.price_type)

        return {
            'view_type': 'form',
            "view_mode": 'form',
            'res_model': 'ir.actions.configuration.wizard',
            'type': 'ir.actions.act_window',
            'target': 'new',
        }

    def action_cancel(self,cr,uid,ids,conect=None):
        return {
            'view_type': 'form',
            "view_mode": 'form',
            'res_model': 'ir.actions.configuration.wizard',
            'type': 'ir.actions.act_window',
            'target':'new',
        }

sale_shop_price_method_policy()


class product_product(osv.osv):
    _name = "product.product"
    _inherit = "product.product"
    _description = "Product"

    def _get_all_values(self, cr, uid, ids, name, arg, context=None):
        res = {}
        product_obj = self.pool.get('product.product')
        pricelist_proxy = self.pool.get('product.pricelist')
        sol_proxy = self.pool.get('sale.order.line')
        res_partner_obj = self.pool.get("res.partner")
        for line in product_obj.browse(cr, uid, ids, context=context):
            c_price1, c_price2, list_price1, last_price, qty_last = 0.0, 0.0, 0.0, 0.0, 0.0

            partner = line.pi_partner_id.id
            pricelist = line.pi_pricelist_id.id
            qty1 = line.pi_quantity1
            qty2 = line.pi_quantity2
            if pricelist != False:
                values = pricelist_proxy.price_get(cr, uid, [pricelist], line.id, qty1, partner)
                c_price1 = values[pricelist]

                values = pricelist_proxy.price_get(cr, uid, [pricelist], line.id, qty2, partner)
                c_price2 = values[pricelist]

                sale_price_list = res_partner_obj.browse(cr, uid, partner).property_product_pricelist.id
                values = pricelist_proxy.price_get(cr, uid, [sale_price_list], line.id, qty1, partner)
                list_price1 = values[sale_price_list]

                read_ids = sol_proxy.search(cr,uid,[('product_id','=',line.id)])
                last_price, qty_last = 0.0, 0.0
                if read_ids:
                    last_id = max(read_ids)
                    last_brw = sol_proxy.browse(cr,uid,[last_id],context=context)
                    if line.id == last_brw[0].product_id.id:
                        last_price = last_brw[0].price_unit
                        qty_last = last_brw[0].product_uom_qty
            res[line.id] = {}

            res[line.id]['pi_customer_prices1'] = c_price1 or 0.0
            res[line.id]['pi_customer_prices2'] = c_price2 or 0.0
            res[line.id]['pi_list_prices1'] = list_price1 or 0.0
            res[line.id]['pi_last_customer_prices']= last_price or 0.0
            res[line.id]['pi_qty_buyed']= qty_last or 0.0

        return res

    def button_dummy(self, cr, uid, ids, context=None):
        return True

    _columns = {
        'pi_quantity1': fields.float('Quantity 1'),
        'pi_quantity2': fields.float('Quantity 2'),
        'pi_pricelist_id': fields.many2one('product.pricelist', 'Pricelist'),
        'currency': fields.char('Currency', size=16,readonly=True),
        'pi_partner_id': fields.many2one('res.partner', 'Customer'),
        'pi_customer_prices1': fields.function(_get_all_values, method=True, type='float', string='Customer Price 1',multi='pi_customer_prices1',help='Customer prices for quantity 1'),
        'pi_customer_prices2': fields.function(_get_all_values, method=True, type='float', string='Customer Price 2',multi='pi_customer_prices2',help='Customer prices for quantity 2'),
        'pi_last_customer_prices': fields.function(_get_all_values, method=True, type='float', string='Last Price',multi='pi_last_customer_prices'),
        'pi_qty_buyed': fields.function(_get_all_values, method=True, type='float', string='Last Quantity Bought',multi='pi_qty_buyed',help="Customer's last bought quantity"),
    }
    _defaults = {
        'pi_quantity1': lambda *a: 1.0,
        'pi_quantity2': lambda *a: 5.0,
    }
    
    def onchange_partner_id(self, cr, uid, ids, part):
        if not part:
            return {'value':{'pi_pricelist_id': False}}
        part = self.pool.get('res.partner').browse(cr, uid, part)
        pricelist = part.property_product_pricelist and part.property_product_pricelist.id or False
        currency = self.pool.get('product.pricelist').browse(cr, uid, pricelist).currency_id.name
        return {'value':{'pi_pricelist_id': pricelist,'currency':currency}}
    
    def onchange_pricelist_id(self, cr, uid, ids, pricelist_id):
        if not pricelist_id:
            return {'value':{'pi_currency_id': False}}
        currency = self.pool.get('product.pricelist').browse(cr, uid, pricelist_id).currency_id.name
        return {'value':{'currency':currency}}

product_product()

