# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

from osv import osv, fields

class sale_prize_information(osv.osv_memory):
    _name = 'sale.prize.information'
    _description = 'Sale Prize Information'
    _columns = {
        'pricelist_id': fields.many2one('product.pricelist', 'Pricelist',required= True),
        'partner_id': fields.many2one('res.partner', 'Customer',required= True),
        'product_id': fields.many2one('product.product', 'Product',required= True),
        'quantity1': fields.float('Quantity 1'),
        'quantity2': fields.float('Quantity 2'),
        'currency': fields.char('currency', size=16,readonly=True),
        'customer_prices1':fields.float('Costomer Price 1',help='Customer prices for quantity 1'),
        'customer_prices2':fields.float('Customer Price 2',help='Customer prices for quantity 2'),
        'last_customer_prices':fields.float('Last Price'),
        'list_prices1':fields.float('List Prices',help= 'Price Of Public Pricelist'),
        'qty_buyed':fields.float('Last Quantity Bought',help='Last Costomer Buyed Quantity'),
    }

    _defaults = {
        'quantity1': lambda *a: 1.0,
        'quantity2': lambda *a: 5.0,
    }
    
    def onchange_partner_id(self, cr, uid, ids, part):
        if not part:
            return {'value':{'pi_pricelist_id': False}}
        part = self.pool.get('res.partner').browse(cr, uid, part)
        pricelist = part.property_product_pricelist and part.property_product_pricelist.id or False
        currency = self.pool.get('product.pricelist').browse(cr, uid, pricelist).currency_id.name
        return {'value':{'pricelist_id': pricelist,'currency':currency}}
    
    def onchange_pricelist_id(self, cr, uid, ids, pricelist_id):
        if not pricelist_id:
            return {'value':{'pi_currency_id': False}}
        currency = self.pool.get('product.pricelist').browse(cr, uid, pricelist_id).currency_id.name
        return {'value':{'currency':currency}}

    def set_default(self, cr, uid, ids, context=None):
        pricelist_proxy = self.pool.get('product.pricelist')
        sol_proxy = self.pool.get('sale.order.line')
        res_partner_proxy = self.pool.get("res.partner")
        form = self.read(cr, uid, ids, [])[0]
        partner = form['partner_id']
        pricelist = form['pricelist_id']
        product = form['product_id']
        qty1 = form['quantity1']
        qty2 = form['quantity2']

        # Get the prices of the product
        values = pricelist_proxy.price_get(cr, uid, [pricelist], product, qty1, partner)
        c_price1 = values[pricelist]

        values = pricelist_proxy.price_get(cr, uid, [pricelist], product, qty2, partner)
        c_price2 = values[pricelist]

        sale_price_list = res_partner_proxy.browse(cr, uid, partner).property_product_pricelist.id
        values = pricelist_proxy.price_get(cr, uid, [sale_price_list], product, qty1, partner)
        list_price1 = values[sale_price_list]

        read_ids = sol_proxy.search(cr,uid,[('product_id','=',product)])
        last_price, qty_last = 0.0, 0.0
        if read_ids:
            last_id = max(read_ids)
            last_brw = sol_proxy.browse(cr,uid,[last_id],context=context)

            if product == last_brw[0].product_id.id:
                last_price = last_brw[0].price_unit
                qty_last = last_brw[0].product_uom_qty

        values = {
            'customer_prices1':c_price1,
            'customer_prices2':c_price2,
            'list_prices1':list_price1,
            'last_customer_prices':last_price,
            'qty_buyed':qty_last
        }
        self.write(cr , uid, ids, values, context=context)

sale_prize_information()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
