# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################
from osv import fields, osv
from tools.translate import _

class supplier_cost_optimated_leadtimes(osv.osv_memory):

    _name="supplier.cost.optimated.leadtimes"
    _columns = {
        'partner_id': fields.many2one('res.partner', 'Supplier'),
        'account_purchase_rev': fields.many2one('account.analytic.account', 'Acc. Purchase Revenues', required=True),
        'account_cost_pre_del': fields.many2one('account.analytic.account', 'Acc. Delivery Cost'),
        'date_start': fields.date('From Period', required=True),
        'date_stop': fields.date('To Period', required=True),
        'purchase_rev':fields.float("Purchase Revenue",readonly=True,),
        'economic_no_order':fields.float("Economic Number of orders",readonly=True),
        'cost_per_del':fields.float("Cost Per Delivery",readonly=True),
        'economic_lead_time':fields.float("Economic Lead Time", readonly=True),
        'stock_holding_rate_id':fields.many2one("stock.holding.rate",'Holding Rate',required=True),
    }

    def _compute(self, cr, uid, ids, context=None):

        form = self.read(cr, uid, ids, [], context=context)[0]
        analytic_line_obj = self.pool.get('account.analytic.line')
        stock_hold_rate_obj = self.pool.get("stock.holding.rate")
        date_start = form['date_start']
        date_stop = form['date_stop']

        def get_amount_of_analytic_account(account_id, date_start, date_stop):
            ids = analytic_line_obj.search(cr, uid, [('account_id', '=', account_id),
                                                     ('date', '>=', date_start),
                                                     ('date', '<=', date_stop)])
            return float(sum(obj.amount for obj in analytic_line_obj.browse(cr, uid, ids, context=context)))

        pur_balance = get_amount_of_analytic_account(form['account_purchase_rev'], date_start, date_stop)
        cost_balance = get_amount_of_analytic_account(form['account_cost_pre_del'], date_start, date_stop)

        stock_hold_id = stock_hold_rate_obj.browse(cr, uid, form['stock_holding_rate_id'])
        stock_hold_rate = stock_hold_id.stock_hold_rate
        economic_numer_orders = 0.0
        economic_lead_time = 0.0
        if pur_balance and cost_balance:
            economic_numer_orders = (pur_balance * stock_hold_rate)/ (2 * cost_balance)
            economic_lead_time = 360 / economic_numer_orders

        self.write(cr, uid, ids, {'cost_per_del':cost_balance,'purchase_rev':pur_balance, 'economic_no_order':economic_numer_orders,'economic_lead_time':economic_lead_time})
        return True

supplier_cost_optimated_leadtimes()

# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
