# -*- coding: utf-8 -*-
##############################################################################
#
#    OpenERP, Open Source Management Solution
#    Copyright (C) 2004-2010 Tiny SPRL (<http://tiny.be>).
#
#    This program is free software: you can redistribute it and/or modify
#    it under the terms of the GNU Affero General Public License as
#    published by the Free Software Foundation, either version 3 of the
#    License, or (at your option) any later version.
#
#    This program is distributed in the hope that it will be useful,
#    but WITHOUT ANY WARRANTY; without even the implied warranty of
#    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#    GNU Affero General Public License for more details.
#
#    You should have received a copy of the GNU Affero General Public License
#    along with this program.  If not, see <http://www.gnu.org/licenses/>.
#
##############################################################################

import wizard
import pooler
import time
from tools.translate import _
from tools import config
import math
from datetime import datetime
from dateutil.relativedelta import relativedelta


orderpoint_calculator_form = """<?xml version="1.0"?>
<form string="Order Point Calculator ">
    <field name="warehouse_id"/>
    <field name="location_id"/>      
    <field name="seasonal_id"/>
    <field name="delactive"/>
    <field name="method_qty_calculation" colspan="4"/>
    <group attrs="{'invisible':[('method_qty_calculation','=','his_cons_without_product')]}" colspan="4">
        <separator string="Configuration of Base Consumptions Periods" colspan="4"/>
        <field name="date_start"/>
        <field name="date_stop"/>
    </group>
</form>"""

orderpoint_calculator_fields = {
    'warehouse_id': {
        'string': 'Warehouse', 
        'type': 'many2one', 
        'relation': 'stock.warehouse', 
        'required': True
    },

    'delactive': {
        'string':'Delete Exiting Rules(for this warehosue)', 
        'type':'boolean'
    },

    'location_id': {
        'string': 'Location', 
        'type': 'many2one', 
        'relation': 'stock.location', 
        'required': True
    },

    'seasonal_id': {
        'string': 'Season of Consumption', 
        'type': 'many2one', 
        'relation': 'product.seasonal', 
        'required': True,
        'help':'Use this field as index. Reference index is Main Season with 1.00. If you want to change the orderpoint rules you are able to define new season with a seasonal index. This index should reference to 1.00. For example if you expect - 10% consumption for autumn season in comparition to main season enter 0.90.'
    },

    'date_start': {
        'string':'From Date', 
        'type':'date'
    },

    'date_stop': {
        'string':'To Date', 
        'type':'date',
        'default': lambda *a: time.strftime('%Y-%m-%d')
    },

    'method_qty_calculation':{
        'string':"Method Calculation",
        'type':'selection',
        'selection':[('his_cons_with_product', 'Calculate with historical consumptions of products'),
                     ('his_cons_without_product', 'Calculate with Out historical consumptions of product'),
                     ('hist_cons_and_open_sale', 'Calculate with historical consumptions of product and Open sales orders'),
                    ],'required': True,
        'default': lambda *a:'his_cons_with_product'
    },
}

_messages_form = '''<?xml version="1.0"?>
<form string="Warning">
    <label string="Information Regarding Method of Calculation" colspan="4"/>
    <field name="message" colspan="4" nolabel="1"/>
</form>'''

_message_fields = {
    'message': {'string':'Message' ,'type':'text', 'readonly':True}
}

def _get_default(obj, cr, uid, data, context=None):
    pool = pooler.get_pool(cr.dbname)

    warehouse_id = pool.get('stock.warehouse').search(cr, uid, [])[0]
    seasonal_id = pool.get('product.seasonal').search(cr, uid, [])[0]

    w = pool.get('stock.warehouse').browse(cr,uid,warehouse_id, context)
    location_id = w.lot_stock_id.id

    from_date = (datetime.now() - relativedelta(months=3)).strftime('%Y-%m-%d')

    return {'warehouse_id': warehouse_id, 'location_id':location_id,'date_start':from_date,'seasonal_id':seasonal_id}

def _do_calculate(self, cr, uid, data, context):
        pool = pooler.get_pool(cr.dbname)

        purchase_obj = pool.get('purchase.order')
        sale_obj = pool.get('sale.order')
        product_obj = pool.get('product.product')
        supp_obj = pool.get('product.supplierinfo')
        supp_cost_opt_obj = pool.get('supplier.cost.opti.lead.time')
        min_rule_obj = pool.get('stock.warehouse.orderpoint')
        seasonal_obj = pool.get('product.seasonal')
        ##### Calculation of purchase lead time 
        purchase_lead_time = 0.0
        company = pool.get('res.users').browse(cr, uid, uid, context).company_id
        purchase_lead_time = company.po_lead
        
        ##### get seasonal factor
        seasonal_id = data['form']['seasonal_id']
        seasonal_data = seasonal_obj.browse(cr, uid, seasonal_id)
        seasonal_factor = seasonal_data.seasonal_factor
        
        ##### Calculation of supplier lead time

        for id in data['ids']:
            product_id = product_obj.browse(cr, uid, id, context)
            supplier_lead_time = 0.0
            if product_id:
                for supp in product_id.seller_ids:
                    sup = supp_obj.browse(cr, uid, supp.id, context)
                    supplier_lead_time = sup.delay
                    
            ###### Calculation of average_daily_consumption
            date_start = data['form']['date_start']
            date_stop = data['form']['date_stop']
            average_daily_consumption = 0.0
            sale_line_obj = pool.get("sale.order.line")
            consum_qty = 0.0 
            sale_ids = sale_obj.search(cr,uid,[('date_order','>=',date_start),('date_order','<=',date_stop)])
            if sale_ids:
                for sid in sale_ids:
                    sale_line_id = sale_line_obj.search(cr,uid,[('order_id','=',sid),('product_id','=',product_id.id)])
                    for line_id in sale_line_id:
                        sale_line_data= sale_line_obj.browse(cr,uid,line_id)
                        consum_qty += sale_line_data.product_uom_qty
                       ###here consume quantity is total of selected product
            ##################calculation of date difference between first and last sale order
            sale_ids = sale_obj.search(cr,uid,[('date_order','>=',date_start),('date_order','<=',date_stop)], order='date_order asc')
            if sale_ids:
                start_sale_id = sales_ids[0]
                last_sale_id =  sales_ids[-1]
                first_sale_date = sale_obj.browse(cr,uid,start_sale_id).date_order
                last_sale_date = sale_obj.browse(cr,uid,last_sale_id).date_order
                first_date = time.mktime(time.strptime(first_sale_date,'%Y-%m-%d'))
                last_date = time.mktime(time.strptime(last_sale_date,'%Y-%m-%d'))
                diff_day = (last_date-first_date)/(3600*24)
                if diff_day > 0:
                    average_daily_consumption = consum_qty / diff_day
            ###### Calculation of plan_average_daily_consumption
            plan_average_daily_consumption = 0.0
            if product_id.plan_avg_consume:
                plan_average_daily_consumption = product_id.plan_avg_consume
            
            ####### get location #####
            location_id = data['form']['location_id']
            
            ########## calculation historical consumption of products and open sales orders“
            ### total sale qty with open state
            
            sale_qty = 0.0 
            open_sale_ids = sale_obj.search(cr, uid, [('state','in',['manual','progress'])])
            if sale_ids:
                for sale_id in open_sale_ids:
                    sale_open_line_ids= sale_line_obj.search(cr,uid,[('order_id','=',sale_id),('product_id','=',product_id.id)])
                    for open_line_id in sale_open_line_ids:
                        sale_line_data= sale_line_obj.browse(cr,uid,open_line_id)
                        sale_qty += sale_line_data.product_uom_qty
            ###################
            purchase_ids = purchase_obj.search(cr,uid,[])
            if purchase_ids:
                start_pur_id = min(purchase_ids)
                last_pur_id =  max(purchase_ids)
            
            ##################calculation of date difference between first and last purchase order
                first_purchase_date = purchase_obj.browse(cr,uid,start_pur_id).date_order
                last_purchase_date = purchase_obj.browse(cr,uid,last_pur_id).date_order
                first_date = time.mktime(time.strptime(first_purchase_date,'%Y-%m-%d'))
                last_date = time.mktime(time.strptime(last_purchase_date,'%Y-%m-%d'))
                pur_diff_day = (last_date-first_date)/(3600*24)
            product_max_qty = 0.0 
            product_min_qty = 0.0
            
            if data['form']['method_qty_calculation'] == 'his_cons_with_product':
                product_min_qty = int(math.ceil(average_daily_consumption *  (supplier_lead_time + purchase_lead_time) * seasonal_factor))
                product_max_qty = 2 * product_min_qty
            
            elif data['form']['method_qty_calculation'] == 'his_cons_without_product':
                product_min_qty =  int(math.ceil(plan_average_daily_consumption * (supplier_lead_time + purchase_lead_time) * seasonal_factor))
                product_max_qty = 2 * product_min_qty
            
            else:
                try:
                    product_min_qty =  int(math.ceil((consum_qty + sale_qty) / pur_diff_day * (supplier_lead_time + purchase_lead_time) * seasonal_factor))
                except Exception,e:
                     product_min_qty = 0.0
                product_max_qty = 2 * product_min_qty
            
            if data['form']['delactive']:
                stock_rule_ids = min_rule_obj.search(cr,uid,[('warehouse_id','=',data['form']['warehouse_id']),('product_id','=',product_id.id)])
                for stock_rule_id in stock_rule_ids:
                    min_rule_obj.unlink(cr, uid, [stock_rule_id],context=context)
                    
            mini_stock_rule_id = min_rule_obj.search(cr,uid,[('warehouse_id','=',data['form']['warehouse_id']),('product_id','=',product_id.id)])
            if mini_stock_rule_id:
                for stock_rule_id in mini_stock_rule_id:
                    min_rule_obj.write(cr, uid, [stock_rule_id],{'product_min_qty':product_min_qty,'product_max_qty':product_max_qty,},context=context)
            else:
                name =  pool.get('ir.sequence').get(cr, uid, 'stock.orderpoint.calculator')
                pool.get('stock.warehouse.orderpoint').create(cr, uid, {
                                'name': name,
                                'active': True,
                                'warehouse_id': data['form']['warehouse_id'],
                                'location_id': location_id,
                                'product_id': id,
                                'product_min_qty': product_min_qty,
                                'product_max_qty': product_max_qty,
                                'qty_multiple':1,
                                'product_uom':product_id.uom_id.id,
                                'logic':'max'
                            } )  
        

        return {}

def _get_message(self, cr, uid, data, context):

    if data['form']['method_qty_calculation'] == 'his_cons_with_product':
        data['form']['message'] = _("Are your sure that you have enough historical data to calculate stock orderpoint rules with this methodology, Otherwise you should try the other options, \n Calculate without historical consumption of products or ,Calculate with historical consumption of products and sales orders")
    elif data['form']['method_qty_calculation'] == 'his_cons_without_product':
        data['form']['message'] = _("Your calculation is based on the field plan average daily consumption.\n Are your sure not to calculate with the method , Calculate with historical consumption of products")      
    else:
        data['form']['message'] = _("Are your sure that you have really not enough historical data to calculate stock orderpoint rules? \n Otherwise you should prefer the options , Calculate with historical consumption of products. This results may calculate better results than calculation with this methodology.")
    return data['form']    
    

    
class orderpoint_calculator(wizard.interface):
    states = {
        'init' : {
            'actions' : [_get_default],
            'result' : {'type' : 'form',
                    'arch' : orderpoint_calculator_form,
                    'fields' : orderpoint_calculator_fields,
                    'state' : [('end', 'Cancel'),('calculation', 'Calculation') ]}
        },
         'calculation' : {
            'actions' : [ _do_calculate ],
            'result' : {'type' : 'state',
                    'state' : 'end2'}
            },
       
      'end2': {
            'actions': [ _get_message ],
            'result': {'type': 'form', 'arch': _messages_form,
                'fields': _message_fields,
                'state': (
                    ('end', 'Close'),
                )
            },
        }, 
    }
orderpoint_calculator("orderpoint.calculator")



# vim:expandtab:smartindent:tabstop=4:softtabstop=4:shiftwidth=4:
